@@ -12,7 +12,7 @@ use base 'PAR::Repository::Client';
 
 use Carp qw/croak/;
 
-our $VERSION = '0.17';
+our $VERSION = '0.24';
 
 =head1 NAME
 
@@ -89,15 +89,15 @@ sub fetch_par {
     my $self = shift;
     $self->{error} = undef;
     my $file = shift;
+    #warn "FETCHING FILE: $file";
 
-    $ENV{PAR_TEMP} ||= File::Spec->catdir(File::Spec->tmpdir, 'par');
-    mkdir $ENV{PAR_TEMP}, 0777;
+    my $cache_dir = $self->{cache_dir}; # used to be PAR_TEMP, but now configurable
     %escapes = map { chr($_) => sprintf("%%%02X", $_) } 0..255 unless %escapes;
 
     $file =~ m!/([^/]+)$!;
     my $local_file = (defined($1) ? $1 : $file);
     $local_file =~ s/([^\w\._])/$escapes{$1}/g;
-    $local_file = File::Spec->catfile( $ENV{PAR_TEMP}, $local_file);
+    $local_file = File::Spec->catfile( $self->{cache_dir}, $local_file );
 
     my $timeout = $self->{http_timeout};
     my $old_timeout = $ua->timeout();
@@ -110,11 +110,30 @@ sub fetch_par {
     }
 
     return $local_file if -f $local_file;
-    return;
+    return();
   }
 }
 
 
+sub _fetch_as_data {
+  my $self = shift;
+  $self->{error} = undef;
+  my $file = shift;
+  #warn "FETCHING DATA: $file";
+
+  my $timeout = $self->{http_timeout};
+  my $old_timeout = $ua->timeout();
+  $ua->timeout($timeout) if defined $timeout;
+  my $data = LWP::Simple::get( $file );
+  $ua->timeout($old_timeout) if defined $timeout;
+
+  return $data if defined $data;
+
+  $self->{error} = "Could not get '$file' from repository";
+  return();
+}
+
+
 =head2 validate_repository
 
 Makes sure the repository is valid. Returns the empty list
@@ -234,16 +253,25 @@ sub _dbm_checksums {
   my $url = $self->{uri};
   $url =~ s/\/$//;
 
-  my $file = $self->_fetch_file(
+  # if we're running on a "trust-the-checksums-for-this-long" basis...
+  # ... return if the timeout hasn't elapsed
+  if ($self->{checksums} and $self->{checksums_timeout}) {
+    my $time = time();
+    if ($time - $self->{last_checksums_refresh} < $self->{checksums_timeout}) {
+      return($self->{checksums});
+    }
+  }
+
+  my $data = $self->_fetch_as_data(
     $url.'/'.PAR::Repository::Client::DBM_CHECKSUMS_FILE()
   );
 
-  if (not defined $file) {
+  if (not defined $data) {
     $self->{error} = "Repository does not support checksums";
     return();
   }
 
-  return $self->_parse_dbm_checksums($file);
+  return $self->_parse_dbm_checksums(\$data);
 }
 
 
@@ -295,7 +323,7 @@ Steffen Mueller, E<lt>smueller@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2006-2008 by Steffen Mueller
+Copyright (C) 2006-2009 by Steffen Mueller
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.6 or,
