@@ -13,6 +13,8 @@ sub client_ok {
   my $client = PAR::Repository::Client->new(
     uri => $path,
     verbosity => 3,
+    cache_dir => $ENV{PAR_TEMP},
+    checksums_timeout => 0,
   );
 
   Test::More::isa_ok($client, 'PAR::Repository::Client');
